/* Copyright (C) 2012 Julien "Juju" Savard <juju2143@gmail.com>
   Copyright (C) 2002 W.P. van Paassen - peter@paassen.tmfweb.nl

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <display.h>
#include <display_syscalls.h>
#include <keyboard_syscalls.h>
#include <color.h>
#include "trig.h"

int PRGM_GetKey(void)
{
  unsigned char buffer[12];
  PRGM_GetKey_OS( buffer );
  return ( buffer[1] & 0x0F ) * 10 + ( ( buffer[2] & 0xF0 ) >> 4 );
}

int main(void) {
	int key;
	int cont = 1;

	static short pos1 = 0, pos2 = 0, pos3 = 0, pos4 = 0, tpos1, tpos2, tpos3, tpos4;
	static int aSin[512];
	static color_t colors[256];

        Bdisp_AllClr_VRAM();
        Bdisp_EnableColor(1);
	int x = 0;
	float rad;
	 
	/*create sin lookup table */
	for (int i = 0; i < 512; i++)
	{
		rad =  ((float)i * 0.703125) * 0.0174532; /* 360 / 512 * degree to rad, 360 degrees spread over 512 values to be able to use AND 512-1 instead of using modulo 360*/
		aSin[i] = __sinf(rad) * 1024; /*using fixed point math with 1024 as base*/
	}
	/* create palette */
	for (int i = 0; i < 64; ++i)
	{
	      colors[i] = ((i/2) << 11) + (32 - ((i/2) + 1)); 
	      colors[i+64] = (31 << 11) + (i/2);
	      colors[i+128] = ((32 - ((i/2) + 1))<<11) + (32 - ((i/2) + 1));
	      colors[i+192] = i;
	} 

	while (cont) {
		color_t *p = (color_t*)0xA8000000;
	        key = PRGM_GetKey();
		switch (key) {
			case 47:
			case 48:
				key = 0;
				cont = 0;
				break;
		}
		tpos4 = pos4;
		tpos3 = pos3;
		for(int i = 0; i < LCD_HEIGHT_PX; ++i)
		{
			tpos1 = pos1 + 5;
			tpos2 = pos2 + 3;
			tpos3 &= 511;
			tpos4 &= 511;
			for(int j = 0; j < LCD_WIDTH_PX; ++j)
			{
				tpos1 &= 511;
				tpos2 &= 511;
				x = aSin[tpos1] + aSin[tpos2] + aSin[tpos3] + aSin[tpos4]; /*actual plasma calculation*/
				
				*p++ = colors[128+(x>>4)];

				tpos1 += 5;
				tpos2 += 3;
			}
			tpos4 += 3;
			tpos3 += 1;
		}
		pos1 += 9;
		pos3 += 8;
		Bdisp_PutDisp_DD();
	}
 
	return 0;
}
